%**************************************************************************************%
%                                                                                      %
% Red-Green-Blue to Cyan-Magenta-Yellow-blacK transform                                %
%      This function converts an RGB image to a CMYK image.                            %
%                                                                                      %
% Synopsis:                                                                            %
%      Y=rgb2cmyk(X)                                                                   %
%              X=Continuous tone RGB original image (size MxNx3) of type double.       %
%              Y=Continuous tone CMYK image (size MxNx4) of type double.               %
%                                                                                      %
% Daniel Leo Lau                                                                       %
% Copyright June 12, 2000                                                              %
%                                                                                      %
%**************************************************************************************% 
function Y=rgb2cmyk(X)

[M,N,C]=size(X);
if (C~=3 | max(X(:))>1) error('Image X must be an RGB image of type double!'); end;

Y=zeros(M,N,4);
Y(:,:,1:3)=ones(M,N,C)-X;
for m=1:M
for n=1:N
	Y(m,n,4)=min(Y(m,n,1:3));
end;
end;
Y(:,:,1)=Y(:,:,1)-Y(:,:,4);
Y(:,:,2)=Y(:,:,2)-Y(:,:,4);
Y(:,:,3)=Y(:,:,3)-Y(:,:,4);
return;
