/********************************************************************************/
/*                                                                              */
/* LeNTicular error DiFFusion                                                   */
/*                                                                              */
/*  Synopsis:                                                                   */
/*  	Y = ltndiff(X)                                                          */
/*				X = continuous-tone image of type double                        */
/*				Y = binary halftone of type uint8                               */
/*				If X is a color image, this function halftones each channel     */
/*				independently of the others.                                    */
/*                                                                              */
/* Dr. Daniel L. Lau									                     	*/
/* Copyright July 13, 2007                                                      */
/*											                                    */
/********************************************************************************/ 
#include <math.h>
#include "mex.h"
#define White_Value 		1.0
#define Threshold   		0.0
#define weight_row_radius   1
#define weight_col_radius   1
#define weight_row          3
#define weight_col          3
#define direction		   -1

void set_weights(double *new_weights, double *old_weights)
{
	double frand;

	frand=((double)rand()/(double)RAND_MAX-0.5)*old_weights[2];
	new_weights[2]=old_weights[2]+frand;
	new_weights[8]=old_weights[8]-frand;
	frand=((double)rand()/(double)RAND_MAX-0.5)*old_weights[5];
	new_weights[5]=old_weights[5]+frand;
	new_weights[7]=old_weights[7]-frand;
	return;
}

void ulichney(double *output_image, double *error_image, double *input_image,
              int image_row, int image_col)
{	int m_p, n_p, r, t, d;
	int m, n;
	double weight_matrix[9]={0.0, 0.0, 3.0/16.0, 0.0, 0.0, 5.0/16.0, 0.0, 7.0/16.0, 1.0/16.0};
	double fs_matrix[9]={0.0, 0.0, 3.0/16.0, 0.0, 0.0, 5.0/16.0, 0.0, 7.0/16.0, 1.0/16.0};

	srand(1.0);
	for (m=0; m<image_row; m++){
		d=(int)pow((float)direction, (float)m);
		if (d==1) n=0; else n=image_col-1;
		while(n>=0 && n<image_col){
			set_weights(weight_matrix, fs_matrix);

			output_image[m+n*image_row]=input_image[m+n*image_row] + error_image[m+n*image_row];
			if (output_image[m+n*image_row] >= Threshold) output_image[m+n*image_row]=1.0;
			else output_image[m+n*image_row]=0.0;
			
			error_image[m+n*image_row]=error_image[m+n*image_row]+input_image[m+n*image_row]-output_image[m+n*image_row] * White_Value;
			for (r=0; r<=weight_row_radius; r++){
				for (t=-1*weight_col_radius; t<=weight_col_radius; t++){
					if (r==0 && t<=0) continue;
					m_p=m+r;   if (m_p < 0 || m_p >= image_row) continue;
					n_p=n+d*t; if (n_p < 0 || n_p >= image_col) continue;
					error_image[m_p+n_p*image_row] += error_image[m+n*image_row] * weight_matrix[r+weight_row_radius+(t+weight_col_radius)*weight_row];
					}
				}
			n+=d;
			}
		}
	return;
}

/*******************************************************************************/ 
/* mexFUNCTION                                                                 */
/* Gateway routine for use with MATLAB.                                        */
/*******************************************************************************/ 
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	unsigned char *output_data;
	double *output_image, *error_image, *input_image;
	int m, n, output_dims[2];
	int image_row, image_col;
	int direct;

    /****** Check for errors in user's call to function. ***********************/
        if (nrhs!=1)
                mexErrMsgTxt("Error Diffusion requires exactly one input argument!");
        else if (nlhs>2)
                mexErrMsgTxt("Error Diffusion returns one or two output arguments!");
        else if (!mxIsNumeric(prhs[0]) ||
                  mxIsComplex(prhs[0]) ||
                  mxIsSparse(prhs[0])  ||
                 !mxIsDouble(prhs[0]))
                mexErrMsgTxt("Input X must be a real matrix of type double!");

    /****** Get row and column sizes for input image and weight matrix. ********/
        image_row=mxGetM(prhs[0]);
        image_col=mxGetN(prhs[0]);

    /****** Extract pointers to input data. ************************************/
	input_image=mxGetPr(prhs[0]);

    /****** Create output variables and extract pointers to data. **************/
    output_dims[0]=image_row; output_dims[1]=image_col;
    plhs[0]=mxCreateLogicalArray(2, output_dims);
    output_data=mxGetLogicals(plhs[0]);
    if (nlhs==2){
        plhs[1]=mxCreateDoubleMatrix(image_row, image_col, mxREAL);
        error_image=mxGetPr(plhs[1]);
        }
    else error_image=(double*)mxCalloc(image_row*image_col, sizeof(double));
    output_image=mxCalloc(image_row*image_col, sizeof(double));

    /****** Call Error Diffusion function. *************************************/
	ulichney(output_image, error_image, input_image, image_row, image_col);
	for (n=0; n<image_col; n++){
		for (m=0; m<image_row; m++){
			output_data[m+n*image_row]=output_image[m+n*image_row]>=0.5;
        }
    }
    return;
}
