function H=hxvac(N, g)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  H=hxvac(N, g)
%  HeXagonal Void And Cluster (function)
%     Generates an NxN blue-noise dither pattern representing
%     gray level g on a hexagonal sampling grid.
%  
%  INPUT ARGUMENTS:
% 	N   -> size of the NxN matrix
%   g   -> requested gray level
%	
%  OUTPUT ARGUMENTS:
% 	h   -> NxN dither pattern representing gray level g
%	
%  EXAMPLE:
% 	h = hxvac(256,0.25);
%	disphex(h);
%	
%  Dr. Daniel L. Lau
%  dllau@engr.uky.edu
% 
%  July 19, 2004
%  Copyright 2004 Dr. Daniel L. Lau
% 
%  Last modified on July 19, 2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

H=getinitpattern(N, g);
F=fft2(getfilter(N, 0.2*1/sqrt(g)));

mz=0;
nz=0;
while(1)
	C=real(ifft2(fft2(H).*F));

	Co=C;
	Co(find(H==0))=nan;

	Cz=C;
	Cz(find(H==1))=nan;

	[mo,no]=find(Co==max(max(Co)));
	if (mo==mz) & (no==nz)
		break;
	else
		H(mo,no)=0;
	end; 
	[mz,nz]=find(Cz==min(min(Cz)));
	H(mz,nz)=1;
end;

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function F=getfilter(K, vr)

d=1;
[N,M]=meshgrid(0:(K-1));

u1=[d d/2];
u2=[0 sqrt(3)/2*d];

X=zeros(size(M));
Y=zeros(size(N));

X(:)=u1*[N(:)'; M(:)'];
Y(:)=u2*[N(:)'; M(:)'];

D=[0 0 K K; 0 K 0 K];
Dx=u1*D;
Dy=u2*D;

F=zeros(K,K);
for m=1:K
	for n=1:K
		D=min(sqrt((Dx-X(m,n)).^2+(Dy-Y(m,n)).^2));
		F(m,n)=exp(-D/(2*vr));
	end;
end;
F=F/sum(sum(F));

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function H=getinitpattern(N, g)

H=zeros(N,N);
G=rand(N,N);

num_ones=round(N*N*g);

[Y, I]=sort(G(:));
H(I(1:num_ones))=1;

return;