/****************************************************************************************/
/*                                                                                      */
/* CIRCular SHIFT									                                    */
/*	This function takes a 2-dimensional real, double matrix and shifts its              */
/*	components by M and N.								                                */
/*                                                                                      */
/* Synopsis:                                                                            */
/*      Y=CIRC_SHIFT(X, M, N);								                            */
/*              Y=Output matrix								                            */
/*              X=Input matrix                                                          */ 
/*		M=Integer shift in rows							                                */
/*		N=Integer shift in columns						                                */
/*                                                                                      */
/* Daniel Leo Lau                                                                       */
/* Copyright May 26, 1997                                                               */
/*                                                                                      */
/****************************************************************************************/ 

#include <math.h>
#include "mex.h"

void circular_shift(double output_image[], double input_image[], int M, int N, 
		    int image_row, int image_col)
{
	int m, n;

	for (m=0; m<image_row; m++){
		for (n=0; n<image_col; n++){
			output_image[m+n*image_row]=input_image[((m+image_row-M)%image_row) + 
							        ((n+image_col-N)%image_col)*image_row];
			}
		}
	return;
}

/*******************************************************************************/ 
/* mexFUNCTION                                                                 */
/* Gateway routine for use with MATLAB.                                        */
/*******************************************************************************/ 
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	double *output_image, *input_image;
	int image_row, image_col, M, N;
	int m, n;

    /****** Check for errors in user's call to function. ***********************/
        if (nrhs!=3)
                mexErrMsgTxt("CIRCular_SHIFT requires exactly three input arguments!");
        else if (nlhs>1)
                mexErrMsgTxt("CIRCular_SHIFT returns one output argument!");
        else if (!mxIsNumeric(prhs[0]) ||
                  mxIsComplex(prhs[0]) ||
                  mxIsSparse(prhs[0])  ||
		 !mxIsDouble(prhs[0]))
                mexErrMsgTxt("Input X must be a real matrix of type double!");

    /****** Get row and column sizes for input image ***************************/
    image_row=mxGetM(prhs[0]);
    image_col=mxGetN(prhs[0]);
	M=(int)mxGetScalar(prhs[1])%image_row;
	N=(int)mxGetScalar(prhs[2])%image_col;

    /****** Extract pointers to input data. ************************************/
	input_image=mxGetPr(prhs[0]);

    /****** Create output variables and extract pointers to data. **************/
	plhs[0]=mxCreateDoubleMatrix(image_row, image_col, mxREAL);
	output_image=mxGetPr(plhs[0]);

    /****** Call Circular Shift function. *************************************/
	circular_shift(output_image, input_image, M, N, image_row, image_col);
    return;
}
