%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                   %
% INTeNSiTY                                                         %
%		This function returns the intensity of a given point        %
%		process sample.  If the input point process is that of a    %
%		color halftone, INTENSITY returns the intensity of each     %
%       channel.                                                    %
%                                                                   %
% Synopsis:                                                         %
%       Y=intensity(X)                                              %
%			X=Sample of a point process of type uint8 (logical).    %
%           Y=The intensity of X.                                   %
%                                                                   %
% Dr. Daniel Leo Lau                                                %
% Copyright June 13, 2000                                           %
%                                                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
function Y=intnsty(X)

if (~islogical(X) | ~isa(X, 'uint8'))
	error('Input X must be a point process of type uint8 (logical)!');
end;

[M,N,C]=size(X);
Y=zeros(C,1);
for c=1:C
	Y(c)=sum(sum(X(:,:,c)))/(M*N);
end;

return;
